@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 - 2010 Red Hat, Inc.
+ * (C) Copyright 2008 - 2011 Red Hat, Inc.
  * (C) Copyright 2008 Novell, Inc.
  */
 
@@ -28,15 +28,15 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <gnome-keyring.h>
 
 #include <nm-device.h>
 #include <nm-setting-connection.h>
 #include <nm-setting-gsm.h>
 #include <nm-setting-serial.h>
 #include <nm-setting-ppp.h>
-#include <nm-gsm-device.h>
+#include <nm-device-modem.h>
 #include <nm-utils.h>
+#include <nm-secret-agent.h>
 
 #include "applet.h"
 #include "applet-device-gsm.h"
@@ -65,6 +65,7 @@ typedef struct {
 	NMApplet *applet;
 	NMDevice *device;
 
+	DBusGConnection *bus;
 	DBusGProxy *props_proxy;
 	DBusGProxy *card_proxy;
 	DBusGProxy *net_proxy;
@@ -72,8 +73,6 @@ typedef struct {
 	gboolean quality_valid;
 	guint32 quality;
 	char *unlock_required;
-	char *devid;
-	char *simid;
 	gboolean modem_enabled;
 	MMModemGsmAccessTech act;
 
@@ -91,7 +90,6 @@ typedef struct {
 
 	/* Unlock dialog stuff */
 	GtkWidget *dialog;
-	gpointer keyring_id;
 } GsmDeviceInfo;
 
 static void unlock_dialog_destroy (GsmDeviceInfo *info);
@@ -133,7 +131,7 @@ mobile_wizard_done (MobileWizard *wizard,
 		NMSetting *setting;
 		char *uuid, *id;
 
-		if (method->devtype != NM_DEVICE_TYPE_GSM) {
+		if (method->devtype != NM_DEVICE_MODEM_CAPABILITY_GSM_UMTS) {
 			g_warning ("Unexpected device type (not GSM).");
 			canceled = TRUE;
 			goto done;
@@ -201,7 +199,7 @@ gsm_new_auto_connection (NMDevice *device,
 	info->callback = callback;
 	info->callback_data = callback_data;
 
-	wizard = mobile_wizard_new (NULL, NULL, NM_DEVICE_TYPE_GSM, FALSE,
+	wizard = mobile_wizard_new (NULL, NULL, NM_DEVICE_MODEM_CAPABILITY_GSM_UMTS, FALSE,
 	                            mobile_wizard_done, info);
 	if (wizard) {
 		mobile_wizard_present (wizard);
@@ -210,7 +208,7 @@ gsm_new_auto_connection (NMDevice *device,
 
 	/* Fall back to something */
 	method = g_malloc0 (sizeof (MobileWizardAccessMethod));
-	method->devtype = NM_DEVICE_TYPE_GSM;
+	method->devtype = NM_DEVICE_MODEM_CAPABILITY_GSM_UMTS;
 	method->provider_name = _("GSM");
 	mobile_wizard_done (NULL, FALSE, method, info);
 	g_free (method);
@@ -495,134 +493,24 @@ gsm_get_icon (NMDevice *device,
 }
 
 typedef struct {
-	/* General stuff */
+	SecretsRequest req;
+
 	GtkWidget *dialog;
 	GtkEntry *secret_entry;
 	char *secret_name;
-	NMApplet *applet;
-	NMANewSecretsRequestedFunc callback;
-	gpointer callback_data;
-	NMActiveConnection *active_connection;
-	NMSettingsConnectionInterface *connection;
 } NMGsmSecretsInfo;
 
-
 static void
-secrets_dialog_destroy (gpointer user_data, GObject *finalized)
+free_gsm_secrets_info (SecretsRequest *req)
 {
-	NMGsmSecretsInfo *info = user_data;
+	NMGsmSecretsInfo *info = (NMGsmSecretsInfo *) req;
 
-	gtk_widget_hide (info->dialog);
-	gtk_widget_destroy (info->dialog);
+	if (info->dialog) {
+		gtk_widget_hide (info->dialog);
+		gtk_widget_destroy (info->dialog);
+	}
 
-	g_object_unref (info->connection);
 	g_free (info->secret_name);
-	g_free (info);
-}
-
-static void
-update_cb (NMSettingsConnectionInterface *connection,
-           GError *error,
-           gpointer user_data)
-{
-	if (error) {
-		g_warning ("%s: failed to update connection: (%d) %s",
-		           __func__, error->code, error->message);
-	}
-}
-
-static void
-get_existing_secrets_cb (NMSettingsConnectionInterface *connection,
-                         GHashTable *existing_secrets,
-                         GError *secrets_error,
-                         gpointer user_data)
-{
-	NMGsmSecretsInfo *info = (NMGsmSecretsInfo *) user_data;
-	GHashTable *settings;
-	GError *error = NULL;
-	gboolean save_secret = FALSE;
-	const char *new_secret = NULL;
-
-	if (secrets_error) {
-		error = g_error_copy (secrets_error);
-		goto done;
-	}
-
-	/* Be a bit paranoid */
-	if (connection != info->connection) {
-		g_set_error (&error,
-		             NM_SETTINGS_INTERFACE_ERROR,
-		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
-		             "%s.%d (%s): unexpected reply for wrong connection.",
-		             __FILE__, __LINE__, __func__);
-		goto done;
-	}
-
-	/* Update connection's secrets so we can hash them for sending back to
-	 * NM, and so we can save them all back out to GConf if needed.
-	 */
-	if (existing_secrets) {
-		GHashTableIter iter;
-		gpointer key, value;
-
-		g_hash_table_iter_init (&iter, existing_secrets);
-		while (g_hash_table_iter_next (&iter, &key, &value)) {
-			GError *update_error = NULL;
-			const char *setting_name = key;
-			GHashTable *setting_hash = value;
-
-			/* Keep track of whether or not the user originall saved the secret */
-			if (!strcmp (setting_name, NM_SETTING_GSM_SETTING_NAME)) {
-				if (g_hash_table_lookup (setting_hash, info->secret_name))
-					save_secret = TRUE;
-			}
-
-			if (!nm_connection_update_secrets (NM_CONNECTION (info->connection),
-			                                   setting_name,
-			                                   setting_hash,
-			                                   &update_error)) {
-				g_warning ("%s: error updating connection secrets: (%d) %s",
-				           __func__,
-				           update_error ? update_error->code : -1,
-				           update_error && update_error->message ? update_error->message : "(unknown)");
-				g_clear_error (&update_error);
-			}
-		}
-	}
-
-	/* Now update the secret the user just entered */
-	if (   !strcmp (info->secret_name, NM_SETTING_GSM_PIN)
-	    || !strcmp (info->secret_name, NM_SETTING_GSM_PASSWORD)) {
-		NMSetting *s_gsm;
-
-		s_gsm = (NMSetting *) nm_connection_get_setting (NM_CONNECTION (info->connection),
-		                                                 NM_TYPE_SETTING_GSM);
-		if (s_gsm) {
-			new_secret = gtk_entry_get_text (info->secret_entry);
-			g_object_set (G_OBJECT (s_gsm), info->secret_name, new_secret, NULL);
-		}
-	}
-
-	settings = nm_connection_to_hash (NM_CONNECTION (info->connection));
-	info->callback (info->connection, settings, NULL, info->callback_data);
-	g_hash_table_destroy (settings);
-
-	/* Save secrets back to GConf if the user had entered them into the
-	 * connection originally.  This lets users enter their secret every time if
-	 * they want.
-	 */
-	if (new_secret && save_secret)
-		nm_settings_connection_interface_update (info->connection, update_cb, NULL);
-
- done:
-	if (error) {
-		g_warning ("%s", error->message);
-		info->callback (info->connection, NULL, error, info->callback_data);
-		g_clear_error (&error);
-	}
-
-	nm_connection_clear_secrets (NM_CONNECTION (info->connection));
-	secrets_dialog_destroy (info, NULL);
 }
 
 static void
@@ -630,46 +518,33 @@ get_gsm_secrets_cb (GtkDialog *dialog,
                     gint response,
                     gpointer user_data)
 {
-	NMGsmSecretsInfo *info = (NMGsmSecretsInfo *) user_data;
+	SecretsRequest *req = user_data;
+	NMGsmSecretsInfo *info = (NMGsmSecretsInfo *) req;
+	NMSetting *setting;
 	GError *error = NULL;
 
-	/* Got a user response, clear the NMActiveConnection destroy handler for
-	 * this dialog since this function will now take over dialog destruction.
-	 */
-	g_object_weak_unref (G_OBJECT (info->active_connection), secrets_dialog_destroy, info);
-
 	if (response == GTK_RESPONSE_OK) {
-		const char *hints[2] = { info->secret_name, NULL };
-
-		/* Get existing connection secrets since NM will want those too */
-		if (!nm_settings_connection_interface_get_secrets (info->connection,
-		                                                   NM_SETTING_GSM_SETTING_NAME,
-		                                                   (const char **) &hints,
-		                                                   FALSE,
-		                                                   get_existing_secrets_cb,
-		                                                   info)) {
-			g_set_error (&error,
-			             NM_SETTINGS_INTERFACE_ERROR,
-			             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
-			             "%s.%d (%s): failed to get existing connection secrets",
-			             __FILE__, __LINE__, __func__);
+		setting = nm_connection_get_setting (req->connection, NM_TYPE_SETTING_GSM);
+		if (setting) {
+			g_object_set (G_OBJECT (setting),
+				          info->secret_name, gtk_entry_get_text (info->secret_entry),
+				          NULL);
+		} else {
+			error = g_error_new (NM_SECRET_AGENT_ERROR,
+				                 NM_SECRET_AGENT_ERROR_INTERNAL_ERROR,
+				                 "%s.%d (%s): no GSM setting",
+				                 __FILE__, __LINE__, __func__);
 		}
 	} else {
-		g_set_error (&error,
-		             NM_SETTINGS_INTERFACE_ERROR,
-		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
-		             "%s.%d (%s): canceled",
-		             __FILE__, __LINE__, __func__);
+		error = g_error_new (NM_SECRET_AGENT_ERROR,
+		                     NM_SECRET_AGENT_ERROR_USER_CANCELED,
+		                     "%s.%d (%s): canceled",
+		                     __FILE__, __LINE__, __func__);
 	}
 
-	if (error) {
-		g_warning ("%s", error->message);
-		info->callback (info->connection, NULL, error, info->callback_data);
-		g_error_free (error);
-
-		nm_connection_clear_secrets (NM_CONNECTION (info->connection));
-		secrets_dialog_destroy (info, NULL);
-	}
+	applet_secrets_request_complete_setting (req, NM_SETTING_GSM_SETTING_NAME, error);
+	applet_secrets_request_free (req);
+	g_clear_error (&error);
 }
 
 static void
@@ -699,12 +574,11 @@ pin_entry_changed (GtkEditable *editable, gpointer user_data)
 }
 
 static GtkWidget *
-ask_for_pin (NMDevice *device, GtkEntry **out_secret_entry)
+ask_for_pin (GtkEntry **out_secret_entry)
 {
 	GtkDialog *dialog;
 	GtkWidget *w = NULL, *ok_button = NULL;
 	GtkBox *box = NULL, *vbox = NULL;
-	char *dev_str;
 
 	dialog = GTK_DIALOG (gtk_dialog_new ());
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
@@ -719,12 +593,6 @@ ask_for_pin (NMDevice *device, GtkEntry **out_secret_entry)
 	w = gtk_label_new (_("PIN code is needed for the mobile broadband device"));
 	gtk_box_pack_start (vbox, w, TRUE, TRUE, 0);
 
-	dev_str = g_strdup_printf ("<b>%s</b>", utils_get_device_description (device));
-	w = gtk_label_new (NULL);
-	gtk_label_set_markup (GTK_LABEL (w), dev_str);
-	g_free (dev_str);
-	gtk_box_pack_start (vbox, w, TRUE, TRUE, 0);
-
 	w = gtk_alignment_new (0.5, 0.5, 0, 1.0);
 	gtk_box_pack_start (vbox, w, TRUE, TRUE, 0);
 
@@ -749,74 +617,69 @@ ask_for_pin (NMDevice *device, GtkEntry **out_secret_entry)
 }
 
 static gboolean
-gsm_get_secrets (NMDevice *device,
-                 NMSettingsConnectionInterface *connection,
-                 NMActiveConnection *active_connection,
-                 const char *setting_name,
-                 const char **hints,
-                 NMANewSecretsRequestedFunc callback,
-                 gpointer callback_data,
-                 NMApplet *applet,
-                 GError **error)
+gsm_get_secrets (SecretsRequest *req, GError **error)
 {
-	NMGsmSecretsInfo *secrets_info;
+	NMGsmSecretsInfo *info = (NMGsmSecretsInfo *) req;
 	GtkWidget *widget;
 	GtkEntry *secret_entry = NULL;
 
-	if (!hints || !g_strv_length ((char **) hints)) {
+	applet_secrets_request_set_free_func (req, free_gsm_secrets_info);
+
+	if (!req->hints || !g_strv_length (req->hints)) {
 		g_set_error (error,
-		             NM_SETTINGS_INTERFACE_ERROR,
-		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
+		             NM_SECRET_AGENT_ERROR,
+		             NM_SECRET_AGENT_ERROR_INTERNAL_ERROR,
 		             "%s.%d (%s): missing secrets hints.",
 		             __FILE__, __LINE__, __func__);
 		return FALSE;
 	}
+	info->secret_name = g_strdup (req->hints[0]);
 
-	if (!strcmp (hints[0], NM_SETTING_GSM_PIN)) {
-		GsmDeviceInfo *info = g_object_get_data (G_OBJECT (device), "devinfo");
+	if (!strcmp (info->secret_name, NM_SETTING_GSM_PIN)) {
+		NMDevice *device;
+		GsmDeviceInfo *devinfo;
+
+		device = applet_get_device_for_connection (req->applet, req->connection);
+		if (!device) {
+			g_set_error (error,
+				         NM_SECRET_AGENT_ERROR,
+				         NM_SECRET_AGENT_ERROR_INTERNAL_ERROR,
+				         "%s.%d (%s): failed to find device for active connection.",
+				         __FILE__, __LINE__, __func__);
+			return FALSE;
+		}
+
+		devinfo = g_object_get_data (G_OBJECT (device), "devinfo");
+		g_assert (devinfo);
 
-		g_assert (info);
 		/* A GetSecrets PIN dialog overrides the initial unlock dialog */
-		if (info->dialog)
-			unlock_dialog_destroy (info);
+		if (devinfo->dialog)
+			unlock_dialog_destroy (devinfo);
 
-		widget = ask_for_pin (device, &secret_entry);
-	} else if (!strcmp (hints[0], NM_SETTING_GSM_PASSWORD))
-		widget = applet_mobile_password_dialog_new (device, NM_CONNECTION (connection), &secret_entry);
+		widget = ask_for_pin (&secret_entry);
+	} else if (!strcmp (info->secret_name, NM_SETTING_GSM_PASSWORD))
+		widget = applet_mobile_password_dialog_new (req->connection, &secret_entry);
 	else {
 		g_set_error (error,
-		             NM_SETTINGS_INTERFACE_ERROR,
-		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
+		             NM_SECRET_AGENT_ERROR,
+		             NM_SECRET_AGENT_ERROR_INTERNAL_ERROR,
 		             "%s.%d (%s): unknown secrets hint '%s'.",
-		             __FILE__, __LINE__, __func__, hints[0]);
+		             __FILE__, __LINE__, __func__, info->secret_name);
 		return FALSE;
 	}
+	info->dialog = widget;
+	info->secret_entry = secret_entry;
 
 	if (!widget || !secret_entry) {
 		g_set_error (error,
-		             NM_SETTINGS_INTERFACE_ERROR,
-		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
+		             NM_SECRET_AGENT_ERROR,
+		             NM_SECRET_AGENT_ERROR_INTERNAL_ERROR,
 		             "%s.%d (%s): error asking for GSM secrets.",
 		             __FILE__, __LINE__, __func__);
 		return FALSE;
 	}
 
-	secrets_info = g_malloc0 (sizeof (NMGsmSecretsInfo));
-	secrets_info->callback = callback;
-	secrets_info->callback_data = callback_data;
-	secrets_info->applet = applet;
-	secrets_info->active_connection = active_connection;
-	secrets_info->connection = g_object_ref (connection);
-	secrets_info->secret_name = g_strdup (hints[0]);
-	secrets_info->dialog = widget;
-	secrets_info->secret_entry = secret_entry;
-
-	g_signal_connect (widget, "response", G_CALLBACK (get_gsm_secrets_cb), secrets_info);
-
-	/* Attach a destroy notifier to the NMActiveConnection so we can destroy
-	 * the dialog when the active connection goes away.
-	 */
-	g_object_weak_ref (G_OBJECT (active_connection), secrets_dialog_destroy, secrets_info);
+	g_signal_connect (widget, "response", G_CALLBACK (get_gsm_secrets_cb), info);
 
 	gtk_window_set_position (GTK_WINDOW (widget), GTK_WIN_POS_CENTER_ALWAYS);
 	gtk_widget_realize (GTK_WIDGET (widget));
@@ -827,89 +690,6 @@ gsm_get_secrets (NMDevice *device,
 
 /********************************************************************/
 
-static void
-save_pin_cb (GnomeKeyringResult result, guint32 val, gpointer user_data)
-{
-	if (result != GNOME_KEYRING_RESULT_OK)
-		g_warning ("%s: result %d", (const char *) user_data, result);
-}
-
-static void
-set_pin_in_keyring (const char *devid,
-                    const char *simid,
-                    const char *pin)
-{
-	GnomeKeyringAttributeList *attributes;
-	GnomeKeyringAttribute attr;
-	const char *name;
-	char *error_msg;
-
-	name = g_strdup_printf (_("PIN code for SIM card '%s' on '%s'"),
-	                        simid ? simid : "unknown",
-	                        devid);
-
-	attributes = gnome_keyring_attribute_list_new ();
-	attr.name = g_strdup ("devid");
-	attr.type = GNOME_KEYRING_ATTRIBUTE_TYPE_STRING;
-	attr.value.string = g_strdup (devid);
-	g_array_append_val (attributes, attr);
-
-	if (simid) {
-		attr.name = g_strdup ("simid");
-		attr.type = GNOME_KEYRING_ATTRIBUTE_TYPE_STRING;
-		attr.value.string = g_strdup (simid);
-		g_array_append_val (attributes, attr);
-	}
-
-	error_msg = g_strdup_printf ("Saving PIN code in keyring for devid:%s simid:%s failed",
-	                             devid, simid ? simid : "(unknown)");
-
-	gnome_keyring_item_create (NULL,
-	                           GNOME_KEYRING_ITEM_GENERIC_SECRET,
-	                           name,
-	                           attributes,
-	                           pin,
-	                           TRUE,
-	                           save_pin_cb,
-	                           error_msg,
-	                           (GDestroyNotify) g_free);
-
-	gnome_keyring_attribute_list_free (attributes);
-}
-
-static void
-delete_pin_cb (GnomeKeyringResult result, gpointer user_data)
-{
-	/* nothing to do */
-}
-
-static void
-delete_pins_find_cb (GnomeKeyringResult result, GList *list, gpointer user_data)
-{
-	GList *iter;
-
-	if (result == GNOME_KEYRING_RESULT_OK) {
-		for (iter = list; iter; iter = g_list_next (iter)) {
-			GnomeKeyringFound *found = iter->data;
-
-			gnome_keyring_item_delete (found->keyring, found->item_id, delete_pin_cb, NULL, NULL);
-		}
-	}
-}
-
-static void
-delete_pins_in_keyring (const char *devid)
-{
-	gnome_keyring_find_itemsv (GNOME_KEYRING_ITEM_GENERIC_SECRET,
-	                           delete_pins_find_cb,
-	                           NULL,
-	                           NULL,
-	                           "devid",
-	                           GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
-	                           devid,
-	                           NULL);
-}
-
 static void
 unlock_dialog_destroy (GsmDeviceInfo *info)
 {
@@ -922,14 +702,9 @@ unlock_pin_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
 {
 	GsmDeviceInfo *info = user_data;
 	GError *error = NULL;
-	const char *dbus_error, *msg = NULL, *code1;
+	const char *dbus_error, *msg = NULL;
 
 	if (dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_INVALID)) {
-		if (applet_mobile_pin_dialog_get_auto_unlock (info->dialog)) {
-			code1 = applet_mobile_pin_dialog_get_entry1 (info->dialog);
-			set_pin_in_keyring (info->devid, info->simid, code1);
-		} else
-			delete_pins_in_keyring (info->devid);
 		unlock_dialog_destroy (info);
 		return;
 	}
@@ -1084,11 +859,7 @@ unlock_dialog_new (NMDevice *device, GsmDeviceInfo *info)
 	}
 
 	/* Construct and run the dialog */
-	info->dialog = applet_mobile_pin_dialog_new (title,
-	                                             header,
-	                                             desc,
-	                                             show_pass_label,
-	                                             (unlock_code == UNLOCK_CODE_PIN) ? TRUE : FALSE);
+	info->dialog = applet_mobile_pin_dialog_new (title, header, desc, show_pass_label);
 	g_free (desc);
 	g_return_if_fail (info->dialog != NULL);
 
@@ -1118,9 +889,8 @@ gsm_device_info_free (gpointer data)
 		g_object_unref (info->card_proxy);
 	if (info->net_proxy)
 		g_object_unref (info->net_proxy);
-
-	if (info->keyring_id)
-		gnome_keyring_cancel_request (info->keyring_id);
+	if (info->bus)
+		dbus_g_connection_unref (info->bus);
 
 	if (info->providers)
 		g_hash_table_destroy (info->providers);
@@ -1131,8 +901,6 @@ gsm_device_info_free (gpointer data)
 	if (info->dialog)
 		unlock_dialog_destroy (info);
 
-	g_free (info->devid);
-	g_free (info->simid);
 	g_free (info->op_code);
 	g_free (info->op_name);
 	memset (info, 0, sizeof (GsmDeviceInfo));
@@ -1249,26 +1017,6 @@ parse_op_name (GsmDeviceInfo *info, const char *orig, const char *op_code)
 	return find_provider_for_mcc_mnc (info->providers, orig);
 }
 
-static void
-notify_user_of_gsm_reg_change (GsmDeviceInfo *info)
-{
-	guint32 mb_state = gsm_state_to_mb_state (info);
-
-	if (mb_state == MB_STATE_HOME) {
-		applet_do_notify_with_pref (info->applet,
-		                            _("GSM network."),
-		                            _("You are now registered on the home network."),
-		                            "nm-signal-100",
-		                            PREF_DISABLE_CONNECTED_NOTIFICATIONS);
-	} else if (mb_state == MB_STATE_ROAMING) {
-		applet_do_notify_with_pref (info->applet,
-		                            _("GSM network."),
-		                            _("You are now registered on a roaming network."),
-		                            "nm-signal-100",
-		                            PREF_DISABLE_CONNECTED_NOTIFICATIONS);
-	}
-}
-
 #define REG_INFO_TYPE (dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID))
 #define DBUS_TYPE_G_MAP_OF_VARIANT (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE))
 
@@ -1287,7 +1035,7 @@ reg_info_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
 		if (array->n_values == 3) {
 			value = g_value_array_get_nth (array, 0);
 			if (G_VALUE_HOLDS_UINT (value))
-				new_state = g_value_get_uint (value) + 1;
+				new_state = g_value_get_uint (value);
 
 			value = g_value_array_get_nth (array, 1);
 			if (G_VALUE_HOLDS_STRING (value)) {
@@ -1306,11 +1054,7 @@ reg_info_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
 		g_value_array_free (array);
 	}
 
-	if (info->reg_state != new_state) {
-		info->reg_state = new_state;
-		notify_user_of_gsm_reg_change (info);
-	}
-
+	info->reg_state = new_state + 1;
 	info->op_code = new_op_code;
 	info->op_name = new_op_name;
 
@@ -1355,86 +1099,7 @@ parse_unlock_required (GValue *value)
 }
 
 static void
-keyring_unlock_pin_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
-{
-	GsmDeviceInfo *info = user_data;
-	GError *error = NULL;
-
-	if (!dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_INVALID)) {
-		g_warning ("Failed to auto-unlock devid:%s simid:%s : (%s) %s",
-		           info->devid ? info->devid : "(unknown)",
-		           info->simid ? info->simid : "(unknown)",
-		           dbus_g_error_get_name (error),
-		           error->message);
-		/* Ask the user */
-		unlock_dialog_new (info->device, info);
-		g_clear_error (&error);
-	}
-}
-
-static void
-keyring_pin_check_cb (GnomeKeyringResult result, GList *list, gpointer user_data)
-{
-	GsmDeviceInfo *info = user_data;
-	GList *iter;
-	const char *pin = NULL;
-
-	info->keyring_id = NULL;
-
-	if (result != GNOME_KEYRING_RESULT_OK) {
-		/* No saved PIN, just ask the user */
-		unlock_dialog_new (info->device, info);
-		return;
-	}
-
-	/* Look for a result with a matching "simid" attribute since that's
-	 * better than just using a matching "devid".  The PIN is really tied
-	 * to the SIM, not the modem itself.
-	 */
-	for (iter = list;
-	     info->simid && (pin == NULL) && iter;
-	     iter = g_list_next (iter)) {
-		GnomeKeyringFound *found = iter->data;
-		int i;
-
-		/* Look for a matching "simid" attribute */
-		for (i = 0; (pin == NULL) && i < found->attributes->len; i++) {
-			GnomeKeyringAttribute attr = gnome_keyring_attribute_list_index (found->attributes, i);
-
-			if (   g_strcmp0 (attr.name, "simid") == 0
-			    && attr.type == GNOME_KEYRING_ATTRIBUTE_TYPE_STRING
-			    && g_strcmp0 (attr.value.string, info->simid) == 0) {
-				pin = found->secret;
-				break;
-			}
-		}
-	}
-
-	if (pin == NULL) {
-		/* Fall back to the first result's PIN */
-		pin = ((GnomeKeyringFound *) list->data)->secret;
-		if (pin == NULL) {
-			/* Should never get here */
-			g_warn_if_fail (pin != NULL);
-			unlock_dialog_new (info->device, info);
-			return;
-		}
-	}
-
-	/* Send the PIN code to ModemManager */
-	if (!dbus_g_proxy_begin_call_with_timeout (info->card_proxy, "SendPin",
-	                                           keyring_unlock_pin_reply, info, NULL,
-	                                           15000,  /* 15 seconds */
-	                                           G_TYPE_STRING, pin,
-	                                           G_TYPE_INVALID)) {
-		g_warning ("Failed to auto-unlock devid:%s simid:%s",
-		           info->devid ? info->devid : "(unknown)",
-		           info->simid ? info->simid : "(unknown)");
-	}
-}
-
-static void
-simid_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
+unlock_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
 {
 	GsmDeviceInfo *info = user_data;
 	GError *error = NULL;
@@ -1444,72 +1109,15 @@ simid_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
 	                           G_TYPE_VALUE, &value,
 	                           G_TYPE_INVALID)) {
 		if (G_VALUE_HOLDS_STRING (&value)) {
-			g_free (info->simid);
-			info->simid = g_value_dup_string (&value);
+			g_free (info->unlock_required);
+			info->unlock_required = parse_unlock_required (&value);
+
+			/* Show the unlock dialog if an unlock is now required */
+			if (info->unlock_required)
+				unlock_dialog_new (info->device, info);
 		}
 		g_value_unset (&value);
 	}
-	g_clear_error (&error);
-
-	/* Procure unlock code and apply it if an unlock is now required. */
-	if (info->unlock_required) {
-		/* If we have a device ID ask the keyring for any saved SIM-PIN codes */
-		if (info->devid && (g_strcmp0 (info->unlock_required, "sim-pin") == 0)) {
-			g_warn_if_fail (info->keyring_id == NULL);
-			info->keyring_id = gnome_keyring_find_itemsv (GNOME_KEYRING_ITEM_GENERIC_SECRET,
-			                                              keyring_pin_check_cb,
-			                                              info,
-			                                              NULL,
-			                                              "devid",
-			                                              GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
-			                                              info->devid,
-			                                              NULL);
-		} else {
-			/* Couldn't get a device ID, but unlock required; present dialog */
-			unlock_dialog_new (info->device, info);
-		}
-	}
-
-	check_start_polling (info);
-}
-
-#define MM_DBUS_INTERFACE_MODEM_GSM_CARD "org.freedesktop.ModemManager.Modem.Gsm.Card"
-
-static void
-unlock_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
-{
-	GsmDeviceInfo *info = user_data;
-	GError *error = NULL;
-	GHashTable *props = NULL;
-
-	if (dbus_g_proxy_end_call (proxy, call, &error,
-	                           DBUS_TYPE_G_MAP_OF_VARIANT, &props,
-	                           G_TYPE_INVALID)) {
-		GHashTableIter iter;
-		const char *prop_name;
-		GValue *value;
-
-		g_hash_table_iter_init (&iter, props);
-		while (g_hash_table_iter_next (&iter, (gpointer) &prop_name, (gpointer) &value)) {
-			if ((strcmp (prop_name, "UnlockRequired") == 0) && G_VALUE_HOLDS_STRING (value)) {
-				g_free (info->unlock_required);
-				info->unlock_required = parse_unlock_required (value);
-			}
-
-			if ((strcmp (prop_name, "DeviceIdentifier") == 0) && G_VALUE_HOLDS_STRING (value)) {
-				g_free (info->devid);
-				info->devid = g_value_dup_string (value);
-			}
-		}
-		g_hash_table_destroy (props);
-
-		/* Get SIM card identifier */
-		dbus_g_proxy_begin_call (info->props_proxy, "Get",
-		                         simid_reply, info, NULL,
-		                         G_TYPE_STRING, MM_DBUS_INTERFACE_MODEM_GSM_CARD,
-		                         G_TYPE_STRING, "SimIdentifier",
-		                         G_TYPE_INVALID);
-	}
 
 	g_clear_error (&error);
 	check_start_polling (info);
@@ -1607,13 +1215,8 @@ reg_info_changed_cb (DBusGProxy *proxy,
                      gpointer user_data)
 {
 	GsmDeviceInfo *info = user_data;
-	guint32 new_state = reg_state + 1;
-
-	if (info->reg_state != new_state) {
-		info->reg_state = new_state;
-		notify_user_of_gsm_reg_change (info);
-	}
 
+	info->reg_state = reg_state + 1;
 	g_free (info->op_code);
 	info->op_code = strlen (op_code) ? g_strdup (op_code) : NULL;
 	g_free (info->op_name);
@@ -1682,21 +1285,29 @@ modem_properties_changed (DBusGProxy *proxy,
 static void
 gsm_device_added (NMDevice *device, NMApplet *applet)
 {
-	NMGsmDevice *gsm = NM_GSM_DEVICE (device);
-	AppletDBusManager *dbus_mgr = applet_dbus_manager_get ();
-	DBusGConnection *bus = applet_dbus_manager_get_connection (dbus_mgr);
+	NMDeviceModem *modem = NM_DEVICE_MODEM (device);
 	GsmDeviceInfo *info;
 	const char *udi;
+	DBusGConnection *bus;
+	GError *error = NULL;
 
 	udi = nm_device_get_udi (device);
 	if (!udi)
 		return;
 
+	bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+	if (!bus) {
+		g_warning ("%s: failed to connect to D-Bus: (%d) %s", __func__, error->code, error->message);
+		g_clear_error (&error);
+		return;
+	}
+
 	info = g_malloc0 (sizeof (GsmDeviceInfo));
 	info->applet = applet;
 	info->device = device;
+	info->bus = bus;
 
-	info->props_proxy = dbus_g_proxy_new_for_name (bus,
+	info->props_proxy = dbus_g_proxy_new_for_name (info->bus,
 	                                               "org.freedesktop.ModemManager",
 	                                               udi,
 	                                               "org.freedesktop.DBus.Properties");
@@ -1706,7 +1317,7 @@ gsm_device_added (NMDevice *device, NMApplet *applet)
 		return;
 	}
 
-	info->card_proxy = dbus_g_proxy_new_for_name (bus,
+	info->card_proxy = dbus_g_proxy_new_for_name (info->bus,
 	                                              "org.freedesktop.ModemManager",
 	                                              udi,
 	                                              "org.freedesktop.ModemManager.Modem.Gsm.Card");
@@ -1716,7 +1327,7 @@ gsm_device_added (NMDevice *device, NMApplet *applet)
 		return;
 	}
 
-	info->net_proxy = dbus_g_proxy_new_for_name (bus,
+	info->net_proxy = dbus_g_proxy_new_for_name (info->bus,
 	                                             "org.freedesktop.ModemManager",
 	                                             udi,
 	                                             MM_DBUS_INTERFACE_MODEM_GSM_NETWORK);
@@ -1726,7 +1337,7 @@ gsm_device_added (NMDevice *device, NMApplet *applet)
 		return;
 	}
 
-	g_object_set_data_full (G_OBJECT (gsm), "devinfo", info, gsm_device_info_free);
+	g_object_set_data_full (G_OBJECT (modem), "devinfo", info, gsm_device_info_free);
 
 	/* Registration info signal */
 	dbus_g_object_register_marshaller (nma_marshal_VOID__UINT_STRING_STRING,
@@ -1755,9 +1366,10 @@ gsm_device_added (NMDevice *device, NMApplet *applet)
 	                             info, NULL);
 
 	/* Ask whether the device needs to be unlocked */
-	dbus_g_proxy_begin_call (info->props_proxy, "GetAll",
+	dbus_g_proxy_begin_call (info->props_proxy, "Get",
 	                         unlock_reply, info, NULL,
 	                         G_TYPE_STRING, MM_DBUS_INTERFACE_MODEM,
+	                         G_TYPE_STRING, "UnlockRequired",
 	                         G_TYPE_INVALID);
 
 	/* Ask whether the device is enabled */
@@ -1772,8 +1384,6 @@ gsm_device_added (NMDevice *device, NMApplet *applet)
 	                         G_TYPE_STRING, MM_DBUS_INTERFACE_MODEM_GSM_NETWORK,
 	                         G_TYPE_STRING, "AccessTechnology",
 	                         G_TYPE_INVALID);
-
-	g_object_unref (dbus_mgr);
 }
 
 NMADeviceClass *
@@ -1790,6 +1400,7 @@ applet_device_gsm_get_class (NMApplet *applet)
 	dclass->device_state_changed = gsm_device_state_changed;
 	dclass->get_icon = gsm_get_icon;
 	dclass->get_secrets = gsm_get_secrets;
+	dclass->secrets_request_size = sizeof (NMGsmSecretsInfo);
 	dclass->device_added = gsm_device_added;
 
 	return dclass;
