@@ -19,7 +19,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2011 Red Hat, Inc.
+ * (C) Copyright 2007 - 2010 Red Hat, Inc.
  * (C) Copyright 2007 - 2008 Novell, Inc.
  */
 
@@ -47,8 +47,9 @@
 #include <nm-setting-cdma.h>
 #include <nm-utils.h>
 
+#include <nm-remote-connection.h>
+
 #include "nm-connection-editor.h"
-#include "gconf-helpers.h"
 #include "nma-marshal.h"
 
 #include "ce-page.h"
@@ -128,10 +129,19 @@ ui_to_setting (NMConnectionEditor *editor)
 	autoconnect = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 	g_object_set (G_OBJECT (s_con), NM_SETTING_CONNECTION_AUTOCONNECT, autoconnect, NULL);
 
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (editor->system_checkbutton)))
-		nm_connection_set_scope (editor->connection, NM_CONNECTION_SCOPE_SYSTEM);
-	else
-		nm_connection_set_scope (editor->connection, NM_CONNECTION_SCOPE_USER);
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (editor->all_checkbutton))) {
+		/* Visible to everyone */
+		g_object_set (G_OBJECT (s_con), NM_SETTING_CONNECTION_PERMISSIONS, NULL, NULL);
+	} else {
+		GSList *users = NULL;
+		char buf[75] = "user:";
+
+		/* Visible only to this user */
+		strcat (buf, g_get_user_name ());
+		users = g_slist_append (users, buf);
+		g_object_set (G_OBJECT (s_con), NM_SETTING_CONNECTION_PERMISSIONS, users, NULL);
+		g_slist_free (users);
+	}
 
 	return TRUE;
 }
@@ -157,27 +167,18 @@ update_sensitivity (NMConnectionEditor *editor)
 	 */
 	if (   !nm_setting_connection_get_read_only (s_con)
 	    && editor_is_initialized (editor)) {
-		if (editor->system_settings_can_modify) {
+		if (editor->can_modify) {
 			actionable = ce_polkit_button_get_actionable (CE_POLKIT_BUTTON (editor->ok_button));
 			authorized = ce_polkit_button_get_authorized (CE_POLKIT_BUTTON (editor->ok_button));
 		}
 
-		if (editor->orig_scope == NM_CONNECTION_SCOPE_SYSTEM) {
-			/* If the user cannot ever be authorized to change system connections, and
-			 * the connection is a system connection, we desensitize the entire dialog.
-			 */
-			sensitive = authorized;
-		} else {
-			/* Otherwise, if the connection is originally a user-connection,
-			 * then everything is sensitive except possible the system checkbutton,
-			 * which will be insensitive if the user has no possibility of
-			 * authorizing.
-			 */
-			sensitive = TRUE;
-		}
+		/* If the user cannot ever be authorized to change system connections,
+		 * we desensitize the entire dialog.
+		 */
+		sensitive = authorized;
 	}
 
-	gtk_widget_set_sensitive (GTK_WIDGET (editor->system_checkbutton), actionable && authorized);
+	gtk_widget_set_sensitive (GTK_WIDGET (editor->all_checkbutton), actionable && authorized);
 
 	/* Cancel button is always sensitive */
 	gtk_widget_set_sensitive (GTK_WIDGET (editor->cancel_button), TRUE);
@@ -248,44 +249,40 @@ ok_button_actionable_cb (GtkWidget *button,
 }
 
 static void
-can_modify_changed_cb (NMRemoteSettingsSystem *settings,
-                       GParamSpec *pspec,
-                       NMConnectionEditor *editor)
+permissions_changed_cb (NMClient *client,
+	                    NMClientPermission permission,
+	                    NMClientPermissionResult result,                       
+                        NMConnectionEditor *editor)
 {
+	if (permission != NM_CLIENT_PERMISSION_SETTINGS_MODIFY_SYSTEM)
+		return;
+
+	if (result == NM_CLIENT_PERMISSION_RESULT_YES || result == NM_CLIENT_PERMISSION_RESULT_AUTH)
+		editor->can_modify = TRUE;
+	else
+		editor->can_modify = FALSE;
+
 	connection_editor_validate (editor);
 }
 
 static void
-system_checkbutton_toggled_cb (GtkWidget *widget, NMConnectionEditor *editor)
+all_checkbutton_toggled_cb (GtkWidget *widget, NMConnectionEditor *editor)
 {
-	gboolean use_polkit = TRUE;
-
-	/* The only time the Save button does not need to use polkit is when the
-	 * original connection scope was USER and the "system" checkbutton is
-	 * unchecked.
-	 */
-	if (   !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))
-	    && (editor->orig_scope == NM_CONNECTION_SCOPE_USER))
-		use_polkit = FALSE;
-
-	ce_polkit_button_set_use_polkit (CE_POLKIT_BUTTON (editor->ok_button), use_polkit);
 	connection_editor_validate (editor);
 }
 
+#define NMCE_UI UIDIR "/nm-connection-editor.ui"
+
 static void
 nm_connection_editor_init (NMConnectionEditor *editor)
 {
 	GtkWidget *dialog;
 	GError *error = NULL;
-	const char *objects[] = { "nm-connection-editor", NULL };
 
 	editor->builder = gtk_builder_new ();
 
-	if (!gtk_builder_add_objects_from_file (editor->builder,
-	                                        UIDIR "/nm-connection-editor.ui",
-	                                        (char **) objects,
-	                                        &error)) {
-		g_warning ("Couldn't load builder file " UIDIR "/nm-connection-editor.ui: %s", error->message);
+	if (!gtk_builder_add_from_file (editor->builder, NMCE_UI, &error)) {
+		g_warning ("Couldn't load builder file " NMCE_UI ": %s", error->message);
 		g_error_free (error);
 
 		dialog = gtk_message_dialog_new (NULL, 0,
@@ -301,7 +298,7 @@ nm_connection_editor_init (NMConnectionEditor *editor)
 
 	editor->window = GTK_WIDGET (gtk_builder_get_object (editor->builder, "nm-connection-editor"));
 	editor->cancel_button = GTK_WIDGET (gtk_builder_get_object (editor->builder, "cancel_button"));
-	editor->system_checkbutton = GTK_WIDGET (gtk_builder_get_object (editor->builder, "system_checkbutton"));
+	editor->all_checkbutton = GTK_WIDGET (gtk_builder_get_object (editor->builder, "system_checkbutton"));
 }
 
 static void
@@ -317,6 +314,8 @@ dispose (GObject *object)
 	NMConnectionEditor *editor = NM_CONNECTION_EDITOR (object);
 	GSList *iter;
 
+	if (editor->disposed)
+		goto out;
 	editor->disposed = TRUE;
 
 	g_slist_foreach (editor->initializing_pages, (GFunc) g_object_unref, NULL);
@@ -355,6 +354,10 @@ dispose (GObject *object)
 		editor->builder = NULL;
 	}
 
+	g_signal_handler_disconnect (editor->client, editor->permission_id);
+	g_object_unref (editor->client);
+
+out:
 	G_OBJECT_CLASS (nm_connection_editor_parent_class)->dispose (object);
 }
 
@@ -379,12 +382,11 @@ nm_connection_editor_class_init (NMConnectionEditorClass *klass)
 
 NMConnectionEditor *
 nm_connection_editor_new (NMConnection *connection,
-                          NMRemoteSettingsSystem *settings,
+                          NMClient *client,
                           GError **error)
 {
 	NMConnectionEditor *editor;
 	GtkWidget *hbox;
-	gboolean sensitive = TRUE, use_polkit = FALSE;
 
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), NULL);
 
@@ -394,32 +396,22 @@ nm_connection_editor_new (NMConnection *connection,
 		return NULL;
 	}
 
-	g_object_get (settings,
-	              NM_SETTINGS_SYSTEM_INTERFACE_CAN_MODIFY,
-	              &editor->system_settings_can_modify,
-	              NULL);
-	g_signal_connect (settings,
-	                  "notify::" NM_SETTINGS_SYSTEM_INTERFACE_CAN_MODIFY,
-	                  G_CALLBACK (can_modify_changed_cb),
-	                  editor);
+	editor->client = g_object_ref (client);
 
-	/* If this is a system connection that we can't ever modify,
-	 * set the editor's Save button always insensitive.
-	 */
-	if (nm_connection_get_scope (connection) == NM_CONNECTION_SCOPE_SYSTEM) {
-		sensitive = editor->system_settings_can_modify;
-		use_polkit = TRUE;
-	}
+	editor->can_modify = nm_client_get_permission_result (client, NM_CLIENT_PERMISSION_SETTINGS_MODIFY_SYSTEM);
+	editor->permission_id = g_signal_connect (editor->client,
+	                                          "permission-changed",
+	                                          G_CALLBACK (permissions_changed_cb),
+	                                          editor);
 
 	editor->ok_button = ce_polkit_button_new (_("_Save"),
 	                                          _("Save any changes made to this connection."),
 	                                          _("_Save..."),
 	                                          _("Authenticate to save this connection for all users of this machine."),
 	                                          GTK_STOCK_APPLY,
-	                                          settings,
-	                                          NM_SETTINGS_SYSTEM_PERMISSION_CONNECTION_MODIFY);
+	                                          client,
+	                                          NM_CLIENT_PERMISSION_SETTINGS_MODIFY_SYSTEM);
 	gtk_button_set_use_underline (GTK_BUTTON (editor->ok_button), TRUE);
-	ce_polkit_button_set_use_polkit (CE_POLKIT_BUTTON (editor->ok_button), use_polkit);
 
 	g_signal_connect (editor->ok_button, "actionable",
 	                  G_CALLBACK (ok_button_actionable_cb), editor);
@@ -456,12 +448,10 @@ nm_connection_editor_update_connection (NMConnectionEditor *editor, GError **err
 		return FALSE;
 
 	/* Copy the modified connection to the original connection */
-	settings = nm_connection_to_hash (editor->connection);
+	settings = nm_connection_to_hash (editor->connection, NM_SETTING_HASH_FLAG_ALL);
 	nm_connection_replace_settings (editor->orig_connection, settings, NULL);
 	g_hash_table_destroy (settings);
 
-	nm_connection_set_scope (editor->orig_connection,
-	                         nm_connection_get_scope (editor->connection));
 	return TRUE;
 }
 
@@ -489,10 +479,9 @@ populate_connection_ui (NMConnectionEditor *editor)
 	g_signal_connect_swapped (name, "changed", G_CALLBACK (connection_editor_validate), editor);
 	g_signal_connect_swapped (autoconnect, "toggled", G_CALLBACK (connection_editor_validate), editor);
 
-	g_signal_connect (editor->system_checkbutton, "toggled", G_CALLBACK (system_checkbutton_toggled_cb), editor);
+	g_signal_connect (editor->all_checkbutton, "toggled", G_CALLBACK (all_checkbutton_toggled_cb), editor);
 
-	if (editor->orig_scope == NM_CONNECTION_SCOPE_SYSTEM)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (editor->system_checkbutton), TRUE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (editor->all_checkbutton), TRUE);
 
 	connection_editor_validate (editor);
 }
@@ -562,7 +551,7 @@ page_initialized (CEPage *page, GError *error, gpointer user_data)
 static void request_secrets (GetSecretsInfo *info);
 
 static void
-get_secrets_cb (NMSettingsConnectionInterface *connection,
+get_secrets_cb (NMRemoteConnection *connection,
                 GHashTable *secrets,
                 GError *error,
                 gpointer user_data)
@@ -598,24 +587,12 @@ get_secrets_cb (NMSettingsConnectionInterface *connection,
 static void
 request_secrets (GetSecretsInfo *info)
 {
-	NMSettingsConnectionInterface *connection;
-	gboolean success;
-	GError *error;
-
 	g_return_if_fail (info != NULL);
 
-	connection = NM_SETTINGS_CONNECTION_INTERFACE (info->self->orig_connection);
-	success = nm_settings_connection_interface_get_secrets (connection,
-	                                                        info->setting_name,
-	                                                        NULL,
-	                                                        FALSE,
-	                                                        get_secrets_cb,
-	                                                        info);
-	if (!success) {
-		error = g_error_new_literal (0, 0, _("Failed to update connection secrets due to an unknown error."));
-		get_secrets_cb (connection, NULL, error, info);
-		g_error_free (error);
-	}
+	nm_remote_connection_get_secrets (NM_REMOTE_CONNECTION (info->self->orig_connection),
+	                                  info->setting_name,
+	                                  get_secrets_cb,
+	                                  info);
 }
 
 static void
@@ -625,6 +602,25 @@ get_secrets_for_page (NMConnectionEditor *self,
 {
 	GetSecretsInfo *info;
 
+	if (!setting_name) {
+		/* page doesn't need any secrets */
+		ce_page_complete_init (page, NULL, NULL, NULL);
+		return;
+	}
+
+	/* Try to get secrets from ->orig_connection, because it's the one that
+	 * implements NMSettingsConnectionInterface and can respond to requests for
+	 * its secrets.  ->connection is a plain NMConnection copy of ->orig_connection
+	 * which is the connection that's actually changed when the user clicks stuff.
+	 * When creating importing or creating new connections though, ->orig_connection
+	 * is an NMConnection because it hasn't been exported over D-Bus yet, so we
+	 * can't ask it for secrets, because it doesn't implement NMSettingsConnectionInterface.
+	 */
+	if (!NM_IS_REMOTE_CONNECTION (self->orig_connection)) {
+		ce_page_complete_init (page, setting_name, NULL, NULL);
+		return;
+	}
+
 	info = g_malloc0 (sizeof (GetSecretsInfo));
 	info->self = self;
 	info->page = page;
@@ -652,8 +648,6 @@ get_secrets_for_page (NMConnectionEditor *self,
 	}
 }
 
-#define SECRETS_TAG "secrets-setting-name"
-
 static gboolean
 add_page (NMConnectionEditor *editor,
           CEPageNewFunc func,
@@ -668,17 +662,19 @@ add_page (NMConnectionEditor *editor,
 	g_return_val_if_fail (connection != NULL, FALSE);
 
 	page = (*func) (connection, GTK_WINDOW (editor->window), &secrets_setting_name, error);
-	if (page) {
-		g_object_set_data_full (G_OBJECT (page),
-			                    SECRETS_TAG,
-			                    g_strdup (secrets_setting_name),
-			                    g_free);
+	if (!page)
+		return FALSE;
 
-		editor->initializing_pages = g_slist_append (editor->initializing_pages, page);
-		g_signal_connect (page, "changed", G_CALLBACK (page_changed), editor);
-		g_signal_connect (page, "initialized", G_CALLBACK (page_initialized), editor);
-	}
-	return !!page;
+	editor->initializing_pages = g_slist_prepend (editor->initializing_pages, page);
+	g_signal_connect (page, "changed", G_CALLBACK (page_changed), editor);
+	g_signal_connect (page, "initialized", G_CALLBACK (page_initialized), editor);
+
+	/* Request any secrets the page might require; or if it doesn't want any,
+	 * let the page initialize.
+	 */
+	get_secrets_for_page (editor, page, secrets_setting_name);
+
+	return TRUE;
 }
 
 static gboolean
@@ -689,7 +685,6 @@ nm_connection_editor_set_connection (NMConnectionEditor *editor,
 	NMSettingConnection *s_con;
 	const char *connection_type;
 	gboolean success = FALSE;
-	GSList *iter, *copy;
 
 	g_return_val_if_fail (NM_IS_CONNECTION_EDITOR (editor), FALSE);
 	g_return_val_if_fail (NM_IS_CONNECTION (orig_connection), FALSE);
@@ -701,7 +696,6 @@ nm_connection_editor_set_connection (NMConnectionEditor *editor,
 	editor->connection = nm_connection_duplicate (orig_connection);
 
 	editor->orig_connection = g_object_ref (orig_connection);
-	editor->orig_scope = nm_connection_get_scope (editor->connection);
 	nm_connection_editor_update_title (editor);
 
 	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (editor->connection, NM_TYPE_SETTING_CONNECTION));
@@ -752,38 +746,6 @@ nm_connection_editor_set_connection (NMConnectionEditor *editor,
 		g_warning ("Unhandled setting type '%s'", connection_type);
 	}
 
-	/* After all pages are created, then kick off secrets requests that any
-	 * the pages may need to make; if they don't need any secrets, then let
-	 * them finish initialization.  The list might get modified during the loop
-	 * which is why copy the list here.
-	 */
-	copy = g_slist_copy (editor->initializing_pages);
-	for (iter = copy; iter; iter = g_slist_next (iter)) {
-		CEPage *page = CE_PAGE (iter->data);
-		const char *setting_name = g_object_get_data (G_OBJECT (page), SECRETS_TAG);
-
-		if (!setting_name) {
-			/* page doesn't need any secrets */
-			ce_page_complete_init (page, NULL, NULL, NULL);
-		} else if (!NM_IS_SETTINGS_CONNECTION_INTERFACE (editor->orig_connection)) {
-			/* We want to get secrets using ->orig_connection, since that's the one
-			 * that implements NMSettingsConnectionInterface and thus can actually
-			 * handle secrets requests.  ->connection is a plain NMConnection copy
-			 * of ->orig_connection which is the thing that gets changed when users
-			 * modify anything.  But when creating or importing, ->orig_connection
-			 * will be an NMConnection since the new connection hasn't been added to
-			 * any settings service yet.  So basically, skip requesting secrets if
-			 * the connection can't handle a secrets request.
-			 */
-			ce_page_complete_init (page, setting_name, NULL, NULL);
-		} else {
-			/* Page wants secrets, get them */
-			get_secrets_for_page (editor, page, setting_name);
-		}
-		g_object_set_data (G_OBJECT (page), SECRETS_TAG, NULL);
-	}
-	g_slist_free (copy);
-
 	/* set the UI */
 	recheck_initialization (editor);
 	success = TRUE;
@@ -838,24 +800,6 @@ nm_connection_editor_run (NMConnectionEditor *self)
 	nm_connection_editor_present (self);
 }
 
-void
-nm_connection_editor_save_vpn_secrets (NMConnectionEditor *editor)
-{
-	GSList *iter;
-
-	g_return_if_fail (NM_IS_CONNECTION_EDITOR (editor));
-	g_return_if_fail (nm_connection_get_scope (editor->connection) == NM_CONNECTION_SCOPE_USER);
-
-	for (iter = editor->pages; iter; iter = g_slist_next (iter)) {
-		CEPage *page = CE_PAGE (iter->data);
-
-		if (CE_IS_PAGE_VPN (page)) {
-			ce_page_vpn_save_secrets (page, editor->connection);
-			break;
-		}
-	}
-}
-
 GtkWindow *
 nm_connection_editor_get_window (NMConnectionEditor *editor)
 {
