@@ -31,7 +31,6 @@
 #include <nm-utils.h>
 
 #include "page-wireless.h"
-#include "utils.h"
 
 G_DEFINE_TYPE (CEPageWireless, ce_page_wireless, CE_TYPE_PAGE)
 
@@ -294,7 +293,7 @@ populate_ui (CEPageWireless *self)
 				  NULL);
 
 	if (ssid)
-		utf8_ssid = nm_utils_ssid_to_utf8 ((const char *) ssid->data, ssid->len);
+		utf8_ssid = nm_utils_ssid_to_utf8 (ssid);
 	else
 		utf8_ssid = g_strdup ("");
 	gtk_entry_set_text (priv->ssid, utf8_ssid);
