@@ -21,6 +21,7 @@
  */
 
 #include <string.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <net/ethernet.h>
@@ -45,12 +46,10 @@
 #include <nm-setting-vpn.h>
 #include <nm-setting-ip4-config.h>
 #include <nm-utils.h>
-#include <nm-settings-interface.h>
 
 #include "gconf-helpers.h"
 #include "gconf-upgrade.h"
 #include "utils.h"
-#include "applet.h"
 
 #define S390_OPT_KEY_PREFIX "s390-opt-"
 
@@ -68,6 +67,8 @@
 #define DBUS_TYPE_G_IP6_ROUTE               (dbus_g_type_get_struct ("GValueArray", DBUS_TYPE_G_UCHAR_ARRAY, G_TYPE_UINT, DBUS_TYPE_G_UCHAR_ARRAY, G_TYPE_UINT, G_TYPE_INVALID))
 #define DBUS_TYPE_G_ARRAY_OF_IP6_ROUTE      (dbus_g_type_get_collection ("GPtrArray", DBUS_TYPE_G_IP6_ROUTE))
 
+#define APPLET_PREFS_PATH "/apps/nm-applet"
+
 const char *applet_8021x_cert_keys[] = {
 	"ca-cert",
 	"client-cert",
@@ -83,36 +84,6 @@ const char *vpn_ignore_keys[] = {
 	NULL
 };
 
-static PreKeyringCallback pre_keyring_cb = NULL;
-static gpointer pre_keyring_user_data = NULL;
-
-/* Sets a function to be called before each keyring access */
-void
-nm_gconf_set_pre_keyring_callback (PreKeyringCallback func, gpointer user_data)
-{
-	pre_keyring_cb = func;
-	pre_keyring_user_data = user_data;
-}
-
-void
-pre_keyring_callback (void)
-{
-	GnomeKeyringInfo *info = NULL;
-
-	if (!pre_keyring_cb)
-		return;
-
-	/* Call the pre keyring callback if the keyring is locked or if there
-	 * was an error talking to the keyring.
-	 */
-	if (gnome_keyring_get_info_sync (NULL, &info) == GNOME_KEYRING_RESULT_OK) {
-		if (gnome_keyring_info_get_is_locked (info))
-			(*pre_keyring_cb) (pre_keyring_user_data);
-		gnome_keyring_info_free (info);
-	} else
-		(*pre_keyring_cb) (pre_keyring_user_data);
-}
-
 
 gboolean
 nm_gconf_get_int_helper (GConfClient *client,
@@ -1988,24 +1959,13 @@ nm_gconf_add_keyring_item (const char *connection_uuid,
 	g_return_if_fail (setting_key != NULL);
 	g_return_if_fail (secret != NULL);
 
-	display_name = g_strdup_printf ("Network secret for %s/%s/%s",
-	                                connection_name,
-	                                setting_name,
-	                                setting_key);
-
-	attrs = gnome_keyring_attribute_list_new ();
-	gnome_keyring_attribute_list_append_string (attrs,
-	                                            KEYRING_UUID_TAG,
-	                                            connection_uuid);
-	gnome_keyring_attribute_list_append_string (attrs,
-	                                            KEYRING_SN_TAG,
-	                                            setting_name);
-	gnome_keyring_attribute_list_append_string (attrs,
-	                                            KEYRING_SK_TAG,
-	                                            setting_key);
-
-	pre_keyring_callback ();
-
+	attrs = utils_create_keyring_add_attr_list (NULL,
+	                                            connection_uuid,
+	                                            connection_name,
+	                                            setting_name,
+	                                            setting_key,
+	                                            &display_name);
+	g_assert (attrs);
 	ret = gnome_keyring_item_create_sync (NULL,
 	                                      GNOME_KEYRING_ITEM_GENERIC_SECRET,
 	                                      display_name,
@@ -2013,7 +1973,10 @@ nm_gconf_add_keyring_item (const char *connection_uuid,
 	                                      secret,
 	                                      TRUE,
 	                                      &id);
-
+	if (ret != GNOME_KEYRING_RESULT_OK) {
+		g_warning ("Failed to add keyring item (%s/%s/%s/%s): %d",
+		           connection_uuid, connection_name, setting_name, setting_key, ret);
+	}
 	gnome_keyring_attribute_list_free (attrs);
 	g_free (display_name);
 }
@@ -2032,8 +1995,6 @@ keyring_delete_item (const char *connection_uuid,
 	GnomeKeyringResult ret;
 	GList *iter;
 
-	pre_keyring_callback ();
-
 	ret = gnome_keyring_find_itemsv_sync (GNOME_KEYRING_ITEM_GENERIC_SECRET,
 	                                      &found_list,
 	                                      KEYRING_UUID_TAG,
@@ -2129,7 +2090,7 @@ write_secret_file (const char *path,
 
 	tmppath = g_malloc0 (strlen (path) + 10);
 	if (!tmppath) {
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Could not allocate memory for temporary file for '%s'",
 		             path);
 		return FALSE;
@@ -2141,7 +2102,7 @@ write_secret_file (const char *path,
 	errno = 0;
 	fd = mkstemp (tmppath);
 	if (fd < 0) {
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Could not create temporary file for '%s': %d",
 		             path, errno);
 		goto out;
@@ -2152,7 +2113,7 @@ write_secret_file (const char *path,
 	if (fchmod (fd, S_IRUSR | S_IWUSR)) {
 		close (fd);
 		unlink (tmppath);
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Could not set permissions for temporary file '%s': %d",
 		             path, errno);
 		goto out;
@@ -2163,7 +2124,7 @@ write_secret_file (const char *path,
 	if (written != len) {
 		close (fd);
 		unlink (tmppath);
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Could not write temporary file for '%s': %d",
 		             path, errno);
 		goto out;
@@ -2174,7 +2135,7 @@ write_secret_file (const char *path,
 	errno = 0;
 	if (rename (tmppath, path)) {
 		unlink (tmppath);
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Could not rename temporary file to '%s': %d",
 		             path, errno);
 		goto out;
@@ -2353,8 +2314,10 @@ write_object (GConfClient *client,
 		 * deleted, but /etc/pki/tls/cert.pem would not.
 		 */
 		standard_file = generate_cert_path (id, objtype->suffix);
-		if (g_file_test (standard_file, G_FILE_TEST_EXISTS))
+		if (g_file_test (standard_file, G_FILE_TEST_EXISTS)) {
 			ignored = unlink (standard_file);
+			if (ignored) {};  /* shut gcc up */
+		}
 		g_free (standard_file);
 
 		/* Delete the key from GConf */
@@ -2378,7 +2341,7 @@ write_object (GConfClient *client,
 
 		new_file = generate_cert_path (id, objtype->suffix);
 		if (!new_file) {
-			g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+			g_set_error (error, 0, 0,
 			             "Could not create file path for %s / %s",
 			             setting_name, objtype->setting_key);
 			return FALSE;
@@ -2393,7 +2356,7 @@ write_object (GConfClient *client,
 			nm_gconf_set_string_helper (client, dir, objtype->setting_key, setting_name, new_file);
 			return TRUE;
 		} else {
-			g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+			g_set_error (error, 0, 0,
 			             "Could not write certificate/key for %s / %s: %s",
 			             setting_name, objtype->setting_key,
 			             (write_error && write_error->message) ? write_error->message : "(unknown)");
@@ -2493,7 +2456,7 @@ write_one_certificate (GConfClient *client,
 	}
 
 	if (!handled) {
-		g_set_error (error, NM_SETTINGS_INTERFACE_ERROR, 0,
+		g_set_error (error, 0, 0,
 		             "Unhandled certificate/private-key item '%s'",
 		             key);
 	}
@@ -2769,6 +2732,7 @@ nm_gconf_set_ignore_ca_cert (const char *uuid, gboolean phase2, gboolean ignore)
 	g_object_unref (client);
 }
 
+#if 0
 static char *
 get_always_ask_path (const char *uuid)
 {
@@ -2814,3 +2778,4 @@ nm_gconf_set_8021x_password_always_ask (const char *uuid, gboolean ask)
 	g_object_unref (client);
 }
 
+#endif
