@@ -1164,7 +1164,6 @@ nm_gconf_migrate_0_7_wireless_security (GConfClient *client)
 
 		copy_keyring_to_8021x (client, iter->data, uuid, NM_SETTING_802_1X_PASSWORD);
 		copy_keyring_to_8021x (client, iter->data, uuid, NM_SETTING_802_1X_PIN);
-		copy_keyring_to_8021x (client, iter->data, uuid, NM_SETTING_802_1X_PSK);
 		copy_keyring_to_8021x (client, iter->data, uuid, NMA_PRIVATE_KEY_PASSWORD_TAG);
 		copy_keyring_to_8021x (client, iter->data, uuid, NMA_PHASE2_PRIVATE_KEY_PASSWORD_TAG);
 
@@ -1888,8 +1887,6 @@ nm_gconf_migrate_0_7_ca_cert_ignore (GConfClient *client)
 		                          &ignore_ca_cert);
 		if (ignore_ca_cert)
 			nm_gconf_set_ignore_ca_cert (uuid, FALSE, TRUE);
-		g_free (uuid);
-
 		/* delete old key */
 		unset_one_setting_property (client, dir,
 		                            NM_SETTING_802_1X_SETTING_NAME,
@@ -1918,10 +1915,6 @@ copy_one_cert_value (GConfClient *client,
 {
 	char *path = NULL;
 
-	/* Do nothing if already migrated */
-	if (nm_gconf_key_is_set (client, dir, key, NM_SETTING_802_1X_SETTING_NAME))
-		return;
-
 	if (nm_gconf_get_string_helper (client, dir,
 	                                tag,
 	                                NM_SETTING_802_1X_SETTING_NAME,
