@@ -30,7 +30,6 @@
 #include "gconf-helpers.h"
 #include "eap-method.h"
 #include "wireless-security.h"
-#include "utils.h"
 #include "helpers.h"
 
 struct _EAPMethodTLS {
@@ -246,11 +245,7 @@ private_key_picker_helper (EAPMethod *parent, const char *filename, gboolean cha
 		GtkWindow *parent_window = NULL;
 
 		toplevel = gtk_widget_get_toplevel (parent->ui_widget);
-#if GTK_CHECK_VERSION(2,18,0)
 		if (gtk_widget_is_toplevel (toplevel))
-#else
-		if (GTK_WIDGET_TOPLEVEL (toplevel))
-#endif
 			parent_window = GTK_WINDOW (toplevel);
 
 		dialog = gtk_message_dialog_new (parent_window,
@@ -389,8 +384,7 @@ update_secrets (EAPMethod *parent, NMConnection *connection)
 EAPMethodTLS *
 eap_method_tls_new (WirelessSecurity *ws_parent,
                     NMConnection *connection,
-                    gboolean phase2,
-                    gboolean secrets_only)
+                    gboolean phase2)
 {
 	EAPMethod *parent;
 	EAPMethodTLS *method;
@@ -463,23 +457,6 @@ eap_method_tls_new (WirelessSecurity *ws_parent,
 	                  (GCallback) show_toggled_cb,
 	                  method);
 
-	if (secrets_only) {
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_tls_identity_entry"));
-		gtk_widget_set_sensitive (widget, FALSE);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_tls_user_cert_label"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_tls_user_cert_button"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_tls_private_key_label"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_tls_private_key_button"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_tls_ca_cert_label"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_tls_ca_cert_button"));
-		gtk_widget_hide (widget);
-	}
-
 	return method;
 }
 
